----------------------------------------------------------------------------------
-- Simulacija modula za generiranje signala HSYNC - test bench
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;



entity hsyncModule_tb is
--  Port ( );
end hsyncModule_tb;

architecture Behavioral of hsyncModule_tb is
    ----------------------------------------------------------------------------------
    -- KONSTANTE
    ----------------------------------------------------------------------------------
    -- predpostavljamo uro s frekvenco 25 MHz
    constant clockPeriod : time := 40ns;
    
    ----------------------------------------------------------------------------------
    -- KOMPONENTE
    ----------------------------------------------------------------------------------
    -- unit under test: hsyncModule 
    component hsyncModule is 
        Port(
            clk  : in  std_logic;
            reset: in  std_logic;
            hsync: out std_logic
        );
    end component;
    
    
    ----------------------------------------------------------------------------------
    -- NOTRANJI SIGNALI
    ----------------------------------------------------------------------------------
    -- Signali, ki povezujejo vhode UUT z generatorjem stimulusov
    signal clk_sim, reset_sim : std_logic := '0';
    -- Signali, ki povezujejo izhode UUT z osciloskopom (waveform)
    signal hsync_sim : std_logic := '0';
    
begin
    -- instanciacija modula, ki ga testiramo
    UUT: hsyncModule 
        port map(
            clk   => clk_sim,
            reset => reset_sim,
            hsync => hsync_sim
        );

    -- Generator stimulusov (draljajev) = scenarij poteka signalov 
    -- 1. Ura 
    clk_stimuli: process
    begin 
        clk_sim <= '0';
        wait for clockPeriod/2;
        clk_sim <= '1';
        wait for clockPeriod/2;
        -- alternativa:
        --clk_sim <= not clk_sim;
        --wait for clockPeriod/2;
    end process;
    
    -- 2. Reset 
    reset_stimuli: process
    begin 
        -- dolgi reset (za 3 u.p.)
        reset_sim <= '1';        
        wait for clockPeriod*3;
        
        -- reset umaknemo
        reset_sim <= '0';
        -- akaj v neskonnost
        wait;       
    end process;
end Behavioral;
