----------------------------------------------------------------------------------
-- Krmilnik za VGA: generiranje signala HSYNC
-- Create Date: 19.10.2022
-- Modified:    26.10.2022
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

entity hsyncModule is
    Port ( clk   : in  STD_LOGIC;
           reset : in  STD_LOGIC;
           hsync : out STD_LOGIC);
end hsyncModule;

architecture Behavioral of hsyncModule is
    ----------------------------------------------------------------------------------
    -- KONSTANTE
    ----------------------------------------------------------------------------------
    --    constant SP : integer := 96;  -- Sync pulse
    --    constant T  : integer := 800; -- Hsync period time
    -- Za potrebe simulacije smo konstante prilagodili
    constant SP : integer := 3;  -- Sync pulse
    constant T  : integer := 10; -- Hsync period time
    ----------------------------------------------------------------------------------
    -- KOMPONENTE
    ----------------------------------------------------------------------------------
    
    ----------------------------------------------------------------------------------
    -- NOTRANJI SIGNALI
    ----------------------------------------------------------------------------------
    signal count : unsigned(9 downto 0);  -- 10 bitov za vrednost tevca, ki je maks. 799
    signal sync_on, sync_off, q : std_logic;
    signal resetCounter : std_logic;
begin
    ----------------------------------------------------------------------------------
    -- VZPOREDNI DEL
    ----------------------------------------------------------------------------------
    primerjalnik_SP: sync_on <= '1' when count = SP-1 else '0';
    hsync <= q;
    resetCounter <= reset or sync_off;
    ----------------------------------------------------------------------------------
    -- ZAPOREDNI DEL = PROCESI
    ----------------------------------------------------------------------------------
        
    Counter: process (clk)
    begin
       if rising_edge(clk) then
          if resetCounter='1' then
             count <= (others => '0'); -- postavi count na '0'
          else
             count <= count + 1;
          end if;
       end if;
    end process;
    
    
    -- Namenoma smo uporabili proces, da pokaemo,
    -- da se odloitveno vezje lahko opie tudi v procesu
    primerjalnik_T: process (count)
    begin
        if count = T-1 then
            sync_off <= '1';
        else 
            sync_off <= '0';
        end if;
    end process;

    JK_pomnilna_celica: process (clk)
    begin
        if rising_edge(clk) then 
            if reset = '1' then 
                q <= '0';
            else
                if sync_on = '1' and sync_off = '0' then 
                    q <= '1';
                elsif sync_on = '0' and sync_off = '1' then 
                    q <= '0';
                elsif sync_on = '1' and sync_off = '1' then 
                    q <= not q;
                else
                    q <= q;
                end if;
            end if;
        end if;
    end process;

end Behavioral;
